from typing import List

from sziszapangma.core.alignment.alignment_calculator import \
    AlignmentCalculator
from sziszapangma.core.alignment.alignment_processing_step import \
    AlignmentProcessingStep
from sziszapangma.core.alignment.alignment_step import AlignmentStep
from sziszapangma.core.alignment.distance_matrix_calculator import \
    BinaryDistanceCalculator, DistanceCalculator, CosineDistanceCalculator
from sziszapangma.core.alignment.step_words import StepWords
from sziszapangma.core.transformer.embedding_transformer import \
    EmbeddingTransformer


class AlignmentEmbeddingCalculator(AlignmentCalculator):
    _distance_calculator: DistanceCalculator

    def __init__(self, embedding_transformer: EmbeddingTransformer):
        super().__init__(BinaryDistanceCalculator())
        self._embedding_transformer = embedding_transformer
        self._distance_calculator = CosineDistanceCalculator(
            embedding_transformer)

    def _calculate_distance_for_word_step(
        self,
        step_words: StepWords
    ) -> float:
        return self._distance_calculator.calculate_distance_for_words(
            step_words.reference_word,
            step_words.hypothesis_word
        )

    def _calculate_result_cost_for_step(
        self,
        processing_step: AlignmentProcessingStep
    ) -> float:
        step_words = processing_step.step_words
        return self._calculate_distance_for_word_step(step_words) \
            if processing_step.step_type.is_cross_step() \
            else processing_step.step_cost

    def convert_processing_steps_to_result(
        self,
        processing_steps: List[AlignmentProcessingStep]
    ) -> List[AlignmentStep]:
        return [
            AlignmentStep(step.step_type, step.step_words,
                          self._calculate_result_cost_for_step(step))
            for step in processing_steps
        ]
