from abc import ABC
from typing import List

from sziszapangma.core.alignment.alignment_step import AlignmentStep
from sziszapangma.core.alignment.alignment_util import AlignmentUtil
from sziszapangma.core.wer.span import Span


class WerCalculator(ABC):

    @staticmethod
    def _convert_processing_steps_to_result(
        input_steps: List[AlignmentStep],
        span: Span
    ) -> List[AlignmentStep]:
        indexes_per_steps = AlignmentUtil.get_reference_indexes_per_steps(
            input_steps)
        return [
            AlignmentStep(
                input_steps[step_index].step_type,
                input_steps[step_index].step_words,
                input_steps[step_index].step_cost * span.is_index_belong(
                    indexes_per_steps[step_index])
            )
            for step_index in range(len(input_steps))
        ]

    @staticmethod
    def _calculate_wer(
        steps: List[AlignmentStep],
    ) -> float:
        reference_len = AlignmentUtil.get_reference_length(steps)
        return sum([step.step_cost for step in steps]) / reference_len

    def calculate_wer(
        self,
        steps: List[AlignmentStep]
    ) -> float:
        return self._calculate_wer(steps)

    def calculate_wer_for_spans(
        self,
        steps: List[AlignmentStep],
        spans: List[Span]
    ) -> List[float]:
        return [
            self._calculate_wer(self._convert_processing_steps_to_result(
                input_steps=steps,
                span=span
            ))
            for span in spans
        ]
